#!/bin/bash
########################################################################
# This script takes care of building the runtime environment,
# including bootloader, kernel, filesystem, scripts and utilities.
#
# Assumes binutils, gcc and uClibc for avr32-linux are installed, and
# e2fsimage.
#
# NOTE! If building off the BSP CD-ROM, you need to copy the source
# directory to a writable location (harddrive), then copy the sd-image
# directory into that new location.
#
# Written by AVR Tools <avr32@atmel.com> 2006-12-01
########################################################################

. "`dirname $0`"/buildit.inc.sh
. "`dirname $0`"/linux_headers.inc.sh

if [ $# -ne 3 ]; then
    echo -e "Usage: build-linux-environment <source-dir> <sd_image> <build_dir>"
    echo -e "  <source-dir> - Software folder containing all source code"
    echo -e "                 This is included in the BSP CD at: "
    echo -e "                 /BSP-CD/software"
    echo -e "  <sd-image>   - An linux filesystem reference. "
    echo -e "                 Unpack stk1000-linux-image-reference.tar.gz"
    echo -e "                 which can be found on the BSP CD at: "
    echo -e "                 /BSP-CD/builds/stk1000/"
    echo -e "  <build-dir>  - Output directory"
    echo -e "  "
    echo -e "example: build-linux-environment ../software ../sd-image build"
    exit 1
fi

# Source directory
srcdir="`cd $1 && pwd`"
if [ ! -w $srcdir ]; then
    echo -e "Source dir must be writable. If building from BSP CD, please make"
    echo -e "a copy to a writable location (harddrive)"
    exit 1
fi

# Prepare sd image reference (strip svn info)
sd_image_ref="`cd $2 && pwd`"
rm -rf /tmp/sd_image_ref
mkdir -p /tmp/sd_image_ref
cp -a "$sd_image_ref"/* /tmp/sd_image_ref
sd_image_ref=/tmp/sd_image_ref
for files in `find ${sd_image_ref} | grep '.svn'`; do rm -rf $files; done

# Prepare build directories
BUILDDIR="`cd $3 && pwd`"
if [ ! -d $BUILDDIR ]; then
	mkdir -p ${BUILDDIR} || task_error "Unable to create "${BUILDDIR}
fi
if [ ! -w $BUILDDIR ]; then
    echo -e "The specified build directory is not writable."
    exit 1
fi
mkdir -p ${BUILDDIR}/avr32-linux-image-basic \
	|| task_error "Unable to create linux image directory."
mkdir -p ${BUILDDIR}/avr32-linux-image-stk1000 \
	|| task_error "Unable to create linux image directory."
ROOT_BASIC="`cd ${BUILDDIR}/avr32-linux-image-basic && pwd`"
ROOT_STK1000="`cd ${BUILDDIR}/avr32-linux-image-stk1000 && pwd`"

# Some things are different when building in cygwin
if uname -o | grep -iq cygwin; then
    CYGWIN=yes
    KERNEL_SOURCE=/usr/local/avr32-linux
    KBUILD_HAVE_NLS=no
else
    CYGWIN=no
    KERNEL_SOURCE=/usr/avr32-linux
    KBUILD_HAVE_NLS=yes
fi

# Variables
TIMESTAMP="`date +%Y%m%d`"
LINUX_ATMEL_EXTRAVERSION="-at"${LINUX_ATMEL_VERSION}

function build_uboot() {
    task_start "U-boot - unpacking"
    do_pushd u-boot
    rm -rf u-boot-${U_BOOT_VERSION}
    rm -rf u-boot-${U_BOOT_VERSION}_org
    tar jxf u-boot-${U_BOOT_VERSION}.tar.bz2
    task_success

    task_start "U-Boot - patching"
    do_pushd u-boot-${U_BOOT_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1\
        || task_error "patch $i failed. See patch.out"; done
    do_popd
    task_success

    task_start "U-boot - config and make"
    do_pushd u-boot-${U_BOOT_VERSION}
    make atstk1002_config > make.config.out 2>&1
    # NOTE: Can't make with - j here, so we don't use the do_make function.
    make > make.out 2>&1
    cp u-boot.bin "$BUILDDIR"/u-boot.bin
    do_popd
    do_popd
    task_success
}

function build_flash_upgrade() {
    task_start "flash upgrade utility (u-boot upgrade)"
    do_pushd flash-upgrade
    make clean > /dev/null
    # First check for u-boot.bin in build dir
    if [ -f "$BUILDDIR"/u-boot.bin ]; then
	cp "$BUILDDIR"/u-boot.bin .
    else
	# If not check in u-boot build dir
        if [ -f ../u-boot/u-boot-${U_BOOT_VERSION}/u-boot.bin ]; then
	   cp ../u-boot/u-boot-${U_BOOT_VERSION}/u-boot.bin u-boot.bin
        else
	   task_error "Did not find u-boot.bin"
	fi
    fi
    do_make "Make error. See make.out"
    if [ ! -f flash-upgrade.uimg ]; then
	task_error "Unable to find flash-upgrade.uimg"
    fi
    cp flash-upgrade.uimg "$BUILDDIR"/u-boot-upgrade-1.1.4-at1-from-bsp-1.0.uimg
    do_popd
    task_success
}

function build_uclibc(){
    task_start "uClibc - unpacking"
    do_pushd uclibc

    rm -rf uClibc-${UCLIBC_VERSION}
    rm -rf uClibc-${UCLIBC_VERSION}_org
    tar jxf uClibc-${UCLIBC_VERSION}.tar.bz2 || task_error "tar failed."
    task_success

    task_start "uClibc - patching"
    do_pushd uClibc-${UCLIBC_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1; done
    cp ../uClibc-dot-config .config
    do_popd
    task_success

    task_start "uClibc - make oldconfig and make install"
    do_pushd uClibc-${UCLIBC_VERSION}
    make oldconfig > make.oldconfig.out 2>&1 || \
	task_error "make oldconfig failed. See make.oldconfig.out"
    make KERNEL_SOURCE=$KERNEL_SOURCE > make.out 2>&1 \
	|| task_error "make failed. See make.out"
    make install_runtime DESTDIR="$ROOT" > make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_pushd utils
    make install DESTDIR="$ROOT" \
	> make.install.out 2>&1 || \
	task_error "make install failed. See utils/make.install.out"
    do_popd
    do_popd
    do_popd
    task_success
}

function build_kernel() {
    task_start "Linux kernel - unpacking"
    do_pushd linux
    rm -rf linux-${LINUX_VERSION}
    rm -rf linux-${LINUX_VERSION}${LINUX_ATMEL_EXTRAVERSION}
    tar jxf linux-${LINUX_VERSION}.tar.bz2 || task_error "tar failed."
    mv linux-${LINUX_VERSION} linux-${LINUX_VERSION}${LINUX_ATMEL_EXTRAVERSION}
    task_success

    task_start "Linux kernel - patching"
    do_pushd linux-${LINUX_VERSION}${LINUX_ATMEL_EXTRAVERSION}

    # Remember Linux kernel source
    AVR32_LINUX_KERNEL_SOURCE="`pwd`"

    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
	|| task_error "patch $i failed."; done
    task_success

    task_start "Linux kernel - make oldconfig and make"

    # Update version number in Makefile
    sed -e "s/EXTRAVERSION =/EXTRAVERSION = ${LINUX_ATMEL_EXTRAVERSION} #/g" \
	Makefile > tmp_Makefile
    mv -f tmp_Makefile Makefile

    make ARCH=avr32 KBUILD_HAVE_NLS=$KBUILD_HAVE_NLS atstk1002_defconfig  \
	> make.oldconfig.out 2>&1 || task_error "See make.oldconfig.out"

    # alternative: use menuconfig (requres ncurses)
    [ -x ../../u-boot/u-boot-${U_BOOT_VERSION}/tools/mkimage ] \
	|| task_error "Couldn't find mkimage from U-Boot"

    PATH="../../u-boot/u-boot-${U_BOOT_VERSION}/tools:$PATH" \
    make ARCH=avr32 CROSS_COMPILE=avr32-linux- \
	KBUILD_HAVE_NLS=$KBUILD_HAVE_NLS $MAKEOPTS \
	> make.out 2>&1 || task_error "make uImage failed. See make.out"
    cp arch/avr32/boot/images/uImage $ROOT/ || \
	task_error "Failed to install kernel into $ROOT"
    make ARCH=avr32 CROSS_COMPILE=avr32-linux- \
       KBUILD_HAVE_NLS=$KBUILD_HAVE_NLS $MAKEOPTS \
       INSTALL_MOD_PATH=$ROOT/ modules_install \
       > make.modules_install.out 2>&1 || \
       task_error "make uImage failed. See make.modules_install.out"
    do_popd
    do_popd
    task_success
}

function build_busybox () {
    task_start "Busybox - unpacking"
    do_pushd busybox
    rm -rf busybox-${BUSYBOX_VERSION}
    tar jxf busybox-${BUSYBOX_VERSION}.tar.bz2 || task_error "tar failed"
    task_success

    task_start "Busybox - patching"
    do_pushd busybox-${BUSYBOX_VERSION}
    cp ../busybox-dot-config .config \
	|| task_error "Busybox .config not found"
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    do_popd
    task_success

    task_start "Busybox - make and make install"
    do_pushd busybox-${BUSYBOX_VERSION}
    make oldconfig > make.oldconfig.out 2>&1 || \
	task_error "make oldconfig failed. See make.oldconfig.out"
    do_make "make failed. See make.out"
    make install PREFIX=${ROOT} > make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_strace () {
    task_start "strace - unpacking"
    do_pushd strace
    rm -rf strace-${STRACE_VERSION}
    tar jxf strace-${STRACE_VERSION}.tar.bz2 || task_error "tar failed"
    task_success

    task_start "strace - patching" 
    do_pushd strace-${STRACE_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done   
    do_popd
    task_success

    task_start "strace - configure, make, install" 
    do_pushd strace-${STRACE_VERSION}
    ./configure --host=avr32-linux --sysconfdir=/etc --prefix=/usr \
	> configure.out 2>&1 || task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make install DESTDIR="$ROOT" > make.install.out 2>&1 \
        || task_error "make install failed. See make.install.out"
    do_popd 
    do_popd
    task_success

}

function build_zlib () {
    task_start "Zlib - unpacking"
    do_pushd zlib
    rm -rf zlib-${ZLIB_VERSION}
    tar zxf zlib-${ZLIB_VERSION}.tar.gz
    task_success

    task_start "Zlib - configure, make, copy library to root"
    do_pushd zlib-${ZLIB_VERSION}
    CC=avr32-linux-gcc CFLAGS="-O2" ./configure -shared \
	--prefix=/usr > configure.out 2>&1 || \
	task_error "configure failed. see configure.out"
    do_make "make failed. See make.out"

    if [ ! -d "$ROOT/usr/lib" ]
    then
    mkdir -p  "$ROOT/usr/lib"
    fi
    cp libz.so.${ZLIB_VERSION} "$ROOT/usr/lib" \
	|| task_error "cp failed."
    ln -sf libz.so.${ZLIB_VERSION} "$ROOT/usr/lib/libz.so.1" \
	|| task_error "cp failed."
    do_popd
    do_popd
    task_success
}

function build_gdbserver () {

    # gdb is part of the AVR32 GNU toolchain, and the source organization
    # differs from the othe AVR32 Linux source packages.
    if [ ! -d gdb ]; then
        echo "WARNING: Skipping GBD since it was not found in "$srcdir
	return
    fi

    task_start "gdbserver - autoconf and configure"
    do_pushd gdb || task_error "no gdb"
    tar -zxf gdb.tar.gz || task_error "no gdb.tar.gz"
    do_pushd gdb/gdbserver
    autoconf
    do_popd
    rm -rf gdbserver-${GDB_VERSION}-build
    do_mkpushd gdbserver-${GDB_VERSION}-build
    CC="avr32-linux-gcc" ../gdb/gdbserver/configure --host=avr32-linux \
	> configure.out 2>&1 || task_error "configure failed. see configure.out"
    task_success

    task_start "gdbserver - make and make install"
    do_make "make failed. See make.out"
    make install prefix="$ROOT/usr" > make.install.out 2>&1 \
        || task_error "make install failed. See make.install.out"
    avr32-linux-strip $ROOT/usr/bin/gdbserver
    export CC="$CC_saved"
    do_popd
    do_popd
    task_success
}

function build_ncurses () {
    task_start "ncurses - unpack"
    do_pushd ncurses
    rm -rf ncurses-${NCURSES_VERSION}
    tar zxf ncurses-${NCURSES_VERSION}.tar.gz
    task_success

    task_start "ncurses - patching"
    do_pushd ncurses-${NCURSES_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "ncurses - configure, make, make install"
    ./configure --host=avr32-linux --prefix=/usr --without-normal \
	--without-debug --with-shared --program-transform-name='' \
	> configure.out 2>&1 || task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make install.libs install.progs DESTDIR="$ROOT" > make.install.out 2>&1 || \
	task_error "make install.libs/progs failed. See make.install.out"
    make -C misc install.data DESTDIR="$ROOT" > make.install.misc.out 2>&1 \
	|| task_error "make install.data failed. See make.install.misc.out"
    make uninstall.includes DESTDIR="$ROOT" >> make.install.out 2>&1 || \
	task_error "make uninstall.includes failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_ftpd () {
    task_start "ftpd - unpacking "
    do_pushd troll-ftpd
    rm -rf troll-ftpd-${TROLL_FTPD_VERSION}
    tar zxf troll-ftpd-${TROLL_FTPD_VERSION}.tar.gz
    task_success

    task_start "ftpd - patching"
    do_pushd troll-ftpd-${TROLL_FTPD_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "ftpd - make and copy to target"
    make CC=avr32-linux-gcc > make.out 2>&1 \
	|| task_error "Make failed. See make.out"
    cp ftpd $ROOT/usr/sbin
    avr32-linux-strip $ROOT/usr/sbin/ftpd
    do_popd
    do_popd
    task_success
}

function build_joe () {
    task_start "joe - unpacking"
    do_pushd joe
    rm -rf joe-${JOE_VERSION}
    tar zxf joe-${JOE_VERSION}.tar.gz
    task_success

    task_start "joe - patching"
    do_pushd joe-${JOE_VERSION}
    for i in `cat ../series `; do patch -p0 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "joe - configure, make, make install"
    CFLAGS=-O2 ./configure --host=avr32-linux --prefix=/usr --sysconfdir=/etc \
	> configure.out 2>&1 \
        || task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make install DESTDIR="$ROOT" > make.install.out 2>&1 \
        || task_error "make install failed. See make.install.out"
    avr32-linux-strip "$ROOT/usr/bin/joe"
    do_popd
    do_popd
    task_success
}

function build_libpng () {
    task_start "libpng - unpacking"
    do_pushd libpng
    rm -rf libpng-${LIBPNG_VERSION}
    tar jxf libpng-${LIBPNG_VERSION}.tar.bz2
    task_success

    task_start "libpng - make and copy to image root"
    do_pushd libpng-${LIBPNG_VERSION}
    ln -s scripts/makefile.linux Makefile
    make libpng12.so.0 libpng.so.3.${LIBPNG_VERSION} \
	CC=avr32-linux-gcc ZLIBINC=../../zlib/zlib-${ZLIB_VERSION} \
	prefix="$ROOT/usr" LIBPATH="$ROOT/usr/lib" \
	> make.out 2>&1 \
	|| task_error "make failed. See make.out"
    avr32-linux-strip libpng12.so.0.* libpng.so.3.*
    ln -sf libpng.so.3.${LIBPNG_VERSION} libpng.so.3
    cp -a libpng12.so* libpng.so.3* "$ROOT/usr/lib" || task_error "cp failed."
    do_popd
    do_popd
    task_success
}

function build_libungif () {
    task_start "libungif - unpacking"
    do_pushd libungif
    rm -rf libungif-${LIBUNGIF_VERSION}
    tar jxf libungif-${LIBUNGIF_VERSION}.tar.bz2
    task_success

    task_start "libungif - configure and make install"
    do_pushd libungif-${LIBUNGIF_VERSION}
    CC=avr32-linux-gcc CFLAGS="-O2" ./configure \
	--target=avr32-linux \
	--prefix=/usr \
	--host=i386-linux > configure.out 2>&1 \
	|| task_error "configure failed. See configure.out"
    make install SUBDIRS=lib DESTDIR="$ROOT" > make.install.out 2>&1 \
	|| task_error "make install failed. See make.install.out"
    # get rid of static libs and header file in the runtime environment
    rm "$ROOT"/usr/lib/libungif.{a,la,so} "$ROOT/usr/include/gif_lib.h"
    do_popd
    do_popd
    task_success
}

function build_libjpeg () {
    if test $CYGWIN == yes; then
        echo "libjpeg won't compile in cygwin. skipping..."
        return
    fi
    task_start "libjpeg - unpacking"
    do_pushd libjpeg
    rm -rf jpeg-${LIBJPEG_VERSION}
    tar jxf jpegsrc.v${LIBJPEG_VERSION}.tar.bz2
    task_success

    task_start "libjpeg - make and copy to image root"
    do_pushd jpeg-${LIBJPEG_VERSION}
    cp ../config.guess . || task_error "failed copying config.guess"
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    CC=avr32-linux-gcc CFLAGS="-O2" ./configure \
        --host=avr32-linux \
        --enable-shared \
        --disable-static \
        --prefix=/usr > configure.out 2>&1 \
	|| task_error "configure failed. See configure.out"
    do_make "make failed. See make.out"
    cp -a .libs/libjpeg.so.* "$ROOT/usr/lib" > make.install.out 2>&1 \
	|| task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_fbv () {
    if test $CYGWIN == yes; then
        echo "fbv has problems compiling in cygwin, skipping..."
       return
    fi

    task_start "fbv - unpacking "
    do_pushd fbv
    rm -rf fbv-${FBV_VERSION}
    tar zxf fbv-${FBV_VERSION}.tar.gz
    task_success

    task_start "fbv - patching"
    do_pushd fbv-${FBV_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "fbv - configure, make and copy to image root"
    ./configure --prefix=/usr \
		--cc=avr32-linux-gcc \
		--libs="-lz -lm" > configure.out 2>&1 \
	|| task_error "configure failed. See configure.out."
    make CC=avr32-linux-gcc CFLAGS=-O2 > make.out 2>&1 \
	|| task_error "make failed. See make.out"
    avr32-linux-strip fbv
    cp fbv "$ROOT/usr/bin"
    do_popd
    do_popd
    task_success
}

function build_fbset () {
    task_start "fbset - unpacking"
    do_pushd fbset
    rm -rf fbset-${FBSET_VERSION}
    tar zxf fbset-${FBSET_VERSION}.tar.gz
    task_success

    task_start "fbset - patching"
    do_pushd fbset-${FBSET_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "fbset - make and copy to image root"
    type -p bison > /dev/null || \
        task_error "bison program not found."
    type -p flex > /dev/null || \
        task_error "flex program not found."


    make CC='avr32-linux-gcc -Wall -O2 -I.' > make.out 2>&1 \
	|| task_error "Make failed. See make.out"
    cp fbset $ROOT/usr/sbin
    avr32-linux-strip $ROOT/usr/sbin/fbset
    do_popd
    do_popd
    task_success
}

function build_alsa_lib () {
    task_start "alsa-lib - unpacking"
    do_pushd alsa-lib
    rm -rf alsa-lib-${ALSA_LIB_VERSION}
    tar jxf alsa-lib-${ALSA_LIB_VERSION}.tar.bz2
    task_success

    task_start "alsa-lib - patching"
    do_pushd alsa-lib-${ALSA_LIB_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "alsa-lib - make and make install"
    ./configure --host=avr32-linux --prefix=/usr > configure.out 2>&1 \
	|| task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make install -C src/conf DESTDIR="$ROOT" > make.install.out 2>&1 \
	|| task_error "make install in src/conf failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_alsa_utils () {
    task_start "alsa-utils - unpacking"
    do_pushd alsa-utils
    rm -rf alsa-utils-${ALSA_UTILS_VERSION}
    tar jxf alsa-utils-${ALSA_UTILS_VERSION}.tar.bz2
    task_success

    task_start "alsa-utils - patching"
    do_pushd alsa-utils-${ALSA_UTILS_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "alsa-utils - configure, make and make install"
    # note: requires libasound
    ./configure CFLAGS='-I/usr/avr32-linux/include/ncurses -O2 -static' \
	--host=avr32-linux --prefix=/usr > configure.out 2>&1 \
	|| task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make install DESTDIR="$ROOT" > make.install.out 2>&1 \
	|| task_error "make install failed. See make.install.out"

    # Add avr32-linux symbolic link in /usr
    do_pushd $ROOT/usr
    ln -s . avr32-linux
    do_popd

    do_popd
    do_popd
    task_success
}

function build_fusion () {
    task_start "fusion - unpacking"
    do_pushd fusion
    rm -rf fusion-${FUSION_VERSION}
    tar zxf linux-fusion-${FUSION_VERSION}.tar.gz
    task_success

    task_start "fusion - patching"
    do_pushd linux-fusion-${FUSION_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "fusion - make and make install"
    # NOTE: need to specify some parameters, so can not use do_make
    make KERNEL_BUILD="${AVR32_LINUX_KERNEL_SOURCE}"  \
	KERNEL_SOURCE="${AVR32_LINUX_KERNEL_SOURCE}"  \
	KERNEL_VERSION=${LINUX_VERSION}${LINUX_ATMEL_EXTRAVERSION} \
	KERNEL_MODLIB=/lib/modules/${LINUX_VERSION}${LINUX_ATMEL_EXTRAVERSION} \
	ARCH=avr32 CROSS_COMPILE=avr32-linux- > make.out 2>&1 \
	|| task_error "make failed. See make.out."
    make install KERNEL_BUILD="${AVR32_LINUX_KERNEL_SOURCE}" \
        KERNEL_SOURCE="${AVR32_LINUX_KERNEL_SOURCE}" \
	KERNEL_VERSION=${LINUX_VERSION}${LINUX_ATMEL_EXTRAVERSION} \
	KERNEL_MODLIB=/lib/modules/${LINUX_VERSION}${LINUX_ATMEL_EXTRAVERSION} \
	ARCH=avr32 CROSS_COMPILE=avr32-linux- \
        DESTDIR="$ROOT" > make.install.out 2>&1 \
	|| task_error "make install failed. See make.install.out."
   do_popd
   do_popd
   task_success
}

function build_freetype () {
    task_start "freetype - unpacking"
    do_pushd freetype
    rm -rf freetype-${FREETYPE_VERSION}
    tar jxf freetype-${FREETYPE_VERSION}.tar.bz2
    task_success

    task_start "freetype - patching"
    do_pushd freetype-${FREETYPE_VERSION}
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "freetype - configure, make and make install"
    CFLAGS=-O2 ./configure --host=avr32-linux --prefix=/usr --sysconfdir=/etc \
	> configure.out 2>&1 || task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make install DESTDIR="$ROOT" > make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_directfb () {
    task_start "DirectFB - unpacking"
    do_pushd directfb
    rm -rf DirectFB-${DIRECTFB_VERSION}
    tar zxf DirectFB-${DIRECTFB_VERSION}.tar.gz
    do_pushd DirectFB-${DIRECTFB_VERSION}
    task_success

    task_start "DirectFB - patching"
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "DirectFB - configure, make and make install"
    CFLAGS=-O2 FREETYPE_CONFIG=/usr/avr32-linux/bin/freetype-config \
        ./configure --host=avr32-linux --prefix=/usr \
        --with-gfxdrivers=none --with-inputdrivers=keyboard,ps2mouse \
        --disable-video4linux --enable-jpeg --enable-png --enable-gif \
        --enable-multi --enable-freetype --disable-sdl  > configure.out 2>&1 \
	|| task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make install DESTDIR="$ROOT" > make.install.out 2>&1 \
	|| task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_dropbear () {
    task_start "dropbear - unpacking"
    do_pushd dropbear
    rm -rf dropbear-${DROPBEAR_VERSION}
    tar zxf dropbear-${DROPBEAR_VERSION}.tar.gz
    do_pushd dropbear-${DROPBEAR_VERSION}
    task_success

    task_start "dropbear - patching"
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "dropbear - configure, make and make install"
    CFLAGS=-O2 ./configure --host=avr32-linux --prefix=/usr \
	> configure.out 2>&1 || task_error "configure failed, see configure.out"
    make PROGRAMS="dropbear dbclient dropbearkey dropbearconvert scp" \
        > make.out 2>&1 || task_error "make failed. See make.out"
    make install DESTDIR="$ROOT" \
	PROGRAMS="dropbear dbclient dropbearkey dropbearconvert scp"\
	> make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_ntp () {
    task_start "ntp - unpacking"
    do_pushd ntp
    rm -rf ntp-${NTP_VERSION}
    tar zxf ntp-${NTP_VERSION}.tar.gz
    do_pushd ntp-${NTP_VERSION}
    task_success

    task_start "ntp - patching"
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "ntp - configure, make and make install"
    CFLAGS=-O2 ./configure --host=avr32-linux --prefix=/usr \
	--sysconfdir=/etc \
	> configure.out 2>&1 || task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make install DESTDIR="$ROOT" > make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_net_tools () {
    task_start "net-tools - unpacking"
    do_pushd net-tools
    rm -rf net-tools-${NET_TOOLS_VERSION}
    tar jxf net-tools-${NET_TOOLS_VERSION}.tar.bz2
    do_pushd net-tools-${NET_TOOLS_VERSION}
    task_success

    task_start "net-tools - patching"
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "net-tools - configure, make and make install"
    make CROSS_COMPILE=avr32-linux- PREFIX=/usr DESTDIR="$ROOT" \
	> make.out 2>&1 || \
        task_error "make failed, see make.out"
    make CROSS_COMPILE=avr32-linux- PREFIX=/usr \
	DESTDIR="$ROOT" installbin > make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_dnsmasq () {
    task_start "dnsmasq - unpacking"
    do_pushd dnsmasq
    rm -rf dnsmasq-${DNSMASQ_VERSION}
    tar zxf dnsmasq-${DNSMASQ_VERSION}.tar.gz
    do_pushd dnsmasq-${DNSMASQ_VERSION}
    task_success

    task_start "dnsmasq - patching"
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "dnsmasq - configure, make and make install"
    make CC=avr32-linux-gcc PREFIX=/usr DESTDIR="$ROOT" \
	> make.out 2>&1 || \
	task_error "make failed, see make.out"
    make CC=avr32-linux-gcc PREFIX=/usr DESTDIR="$ROOT" installbin \
        > make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_pong () {
    task_start "pong - make and make install"
    do_pushd pong
    make CROSS_COMPILE=avr32-linux- PREFIX=/usr DESTDIR="$ROOT" \
	> make.out 2>&1 || \
        task_error "make failed, see make.out"
    make CROSS_COMPILE=avr32-linux- PREFIX=/usr DESTDIR="$ROOT" install \
        > make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_popd
    task_success
}

function build_gawk () {
    task_start "gawk - unpacking"
    do_pushd gawk
    rm -rf gawk-${GAWK_VERSION}
    tar jxf gawk-${GAWK_VERSION}.tar.bz2
    do_pushd gawk-${GAWK_VERSION}
    task_success

    task_start "gawk - patching"
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "gawk - configure, make and make install"
    CFLAGS="-O2" ./configure --host=avr32-linux \
	--prefix=/usr --sysconfdir=/etc \
	--disable-rpath --disable-nls > configure.out 2>&1 \
	|| task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make DESTDIR="$ROOT" install \
        > make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_iptables () {
    task_start "iptables - unpacking"
    do_pushd iptables
    rm -rf iptables-${IPTABLES_VERSION}
    tar jxf iptables-${IPTABLES_VERSION}.tar.bz2
    do_pushd iptables-${IPTABLES_VERSION}
    task_success

    task_start "iptables - make and make install"
    make KERNEL_DIR=${AVR32_LINUX_KERNEL_SOURCE} \
        CC=avr32-linux-gcc DO_IPV6=0 \
        DESTDIR=${ROOT} PREFIX=/usr \
	> make.out 2>&1 || \
        task_error "make failed, see make.out"
    make KERNEL_DIR=${AVR32_LINUX_KERNEL_SOURCE} \
        CC=avr32-linux-gcc DO_IPV6=0 \
        DESTDIR=${ROOT} PREFIX=/usr install \
	> make.install.out 2>&1 || \
        task_error "make install failed, see make.install.out"
    do_popd
    do_popd
    task_success
}

function build_haserl () {
    task_start "haserl - unpacking"
    do_pushd haserl
    rm -rf haserl-${HASERL_VERSION}
    tar jxf haserl-${HASERL_VERSION}.tar.bz2
    do_pushd haserl-${HASERL_VERSION}
    task_success

    task_start "haserl - configure, make and make install"
    CFLAGS="-O2" ./configure --host=avr32-linux \
	--prefix=/usr --sysconfdir=/etc \
	> configure.out 2>&1 \
	|| task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make DESTDIR="$ROOT" install \
        > make.install.out 2>&1 || \
	task_error "make install failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function build_webif () {
    task_start "webif - unpacking"
    do_pushd webif
    rm -rf webif-files-${HASERL_VERSION}
    tar jxf webif-files-${HASERL_VERSION}.tar.bz2
    rm -rf webif-src-${HASERL_VERSION}
    tar jxf webif-src-${HASERL_VERSION}.tar.bz2
    do_pushd webif-files-${HASERL_VERSION}
    task_success

    task_start "webif - install files"
    cp -a etc ${ROOT}
    cp -a usr ${ROOT}
    cp -a www ${ROOT}
    do_popd
    task_success

    task_start "webif - make and install webif-page"
    do_pushd webif-src-${WEBIF_VERSION}
    avr32-linux-gcc -O2 -Wall -o webif-page webif-page.c > \
	make.out 2>&1 \
	|| task_error "make failed. See make.out"
    install webif-page ${ROOT}/usr/bin > \
        install.out 2>&1 \
        || task_error "install failed. See install.out"
    do_popd
    do_popd
    task_success
}

function build_samba () {
    task_start "samba - unpacking"
    do_pushd samba
    rm -rf samba-${SAMBA_VERSION}
    tar jxf samba-${SAMBA_VERSION}.tar.bz2
    do_pushd samba-${SAMBA_VERSION}
    task_success

    task_start "samba - patching"
    for i in `cat ../series `; do patch -p1 < ../$i > patch.out 2>&1 \
        || task_error "patch $i failed. See patch.out"; done
    task_success

    task_start "samba - configure, make and make install"
    CFLAGS="-O2" ./configure --cache-file=config.cache \
	--host=avr32-linux \
	--prefix=/usr --sysconfdir=/etc \
	--enable-shared --enable-static \
	--disable-pie --libdir=/etc/samba \
	--with-privatedir=/etc/samba \
	--with-piddir=/var/run/samba \
	--localstatedir=/var --with-syslog \
	> configure.out 2>&1 \
	|| task_error "configure failed, see configure.out"
    do_make "make failed. See make.out"
    make DESTDIR="$ROOT" installdirs \
        > make.install.out 2>&1 || \
	task_error "make installdirs failed. See make.install.out"
    make DESTDIR="$ROOT" installbin \
        >> make.install.out 2>&1 || \
	task_error "make installbin failed. See make.install.out"
    make DESTDIR="$ROOT" installservers \
        >> make.install.out 2>&1 || \
	task_error "make installservers failed. See make.install.out"
    make DESTDIR="$ROOT" installswat \
        >> make.install.out 2>&1 || \
	task_error "make installservers failed. See make.install.out"
    do_popd
    do_popd
    task_success
}

function create_root () {
    task_start "Creating new root for "$ROOT_IMAGE_NAME

    # Clear content of root
    rm -rf ${ROOT}/*

    # Copy filestructure from reference file system
    cp -a ${ROOT_REF}/* ${ROOT}

    # Set correct permission to ftp folder
    if [[ -d ${ROOT}/home/ftp ]]; then
       chown 50.50 ${ROOT}/home/ftp
       chmod 775 ${ROOT}/home/ftp
    fi

    # Set correct permission to tmp folder
    if [[ -d ${ROOT}/tmp ]]; then
       chown 50.50 ${ROOT}/tmp
       chmod 775 ${ROOT}/tmp
    fi

    # Adjust init to specified development board
    if [[ `echo "$ROOT_IMAGE_NAME" | grep "STK1000"` ]]; then
        if [ -f  ${ROOT}/etc/init.d/rcS-stk1000 ]; then
            rm -rf ${ROOT}/etc/init.d/rcS
            mv ${ROOT}/etc/init.d/rcS-stk1000 ${ROOT}/etc/init.d/rcS
        fi
    fi
    if [[ `echo "$ROOT_IMAGE_NAME" | grep "NGW"` ]]; then
        if [ -f  ${ROOT}/etc/init.d/rcS-ngw ]; then
            rm -rf ${ROOT}/etc/init.d/rcS
            mv ${ROOT}/etc/init.d/rcS-ngw ${ROOT}/etc/init.d/rcS
        fi
    fi

    # Adjust httpd.conf to specified development board
    if [[ `echo "$ROOT_IMAGE_NAME" | grep "STK1000"` ]]; then
        if [ -f  ${ROOT}/etc/httpd-stk1000.conf ]; then
            rm -rf ${ROOT}/etc/httpd.conf
            mv ${ROOT}/etc/httpd-stk1000.conf ${ROOT}/etc/httpd.conf
        fi
    fi
    if [[ `echo "$ROOT_IMAGE_NAME" | grep "NGW"` ]]; then
        if [ -f  ${ROOT}/etc/httpd-ngw.conf ]; then
            rm -rf ${ROOT}/etc/httpd.conf
            mv ${ROOT}/etc/httpd-ngw.conf ${ROOT}/etc/httpd.conf
        fi
    fi

    # Adjust modules to specified development board
    if [[ `echo "$ROOT_IMAGE_NAME" | grep "STK1000"` ]]; then
        if [ -f ${ROOT}/etc/modules-stk1000 ]; then
            rm -rf ${ROOT}/etc/modules
            mv ${ROOT}/etc/modules-stk1000 ${ROOT}/etc/modules
        fi
    fi
    if [[ `echo "$ROOT_IMAGE_NAME" | grep "NGW"` ]]; then
        if [ -f ${ROOT}/etc/modules-stk1000 ]; then
            rm -rf ${ROOT}/etc/modules
            mv ${ROOT}/etc/modules-ngw ${ROOT}/etc/modules
        fi
    fi

    task_success
}

function create_etc_release () {
    task_start "Creating /etc/release for "$ROOT_IMAGE_NAME
    echo $ROOT_IMAGE_NAME", timestamp: "$TIMESTAMP  > "$ROOT/etc/release" \
	|| task_error "Failed to create /etc/release."
    task_success
}


function get_stk1000_web () {
    if [ ! -d ../documentation/stk1000-web ]; then
        task_start "Skipping STK1000 web contents."
	echo
    else
        task_start "Copying STK1000 web contents... "
        cp -a ../documentation/stk1000-web/* ${ROOT}/www
    fi
    task_success
}

function create_image () {
    task_start "Creating runtime image for "$ROOT_IMAGE_NAME

    type -p e2fsimage > /dev/null || \
        task_error "e2fsimage program not found, skipping image creation."
    e2fsimage -f "$ROOT.img" -d "$ROOT" -s $ROOT_IMAGE_SIZE \
	> e2fsimage.out 2>&1 || task_error "Failed to create e2fs image."
    task_success

    task_start "Zip runtime image"
    rm -rf "$ROOT.img.gz"
    gzip -9 "$ROOT.img" || task_error "Failed to compress image."
    task_success
}

start_timer
cd "$srcdir" || task_error "Unable to change to source directory $srcdir."

# Build u-boot: STK1000/STK1002 Standard
build_uboot
build_flash_upgrade

# Build Basic Linux image
# This includes all pacakges used by both the STK1000 and the NGW
ROOT=${ROOT_BASIC}
ROOT_REF=$sd_image_ref
ROOT_IMAGE_NAME="Basic AVR32 Linux image"
ROOT_IMAGE_SIZE=120000
create_root
build_uclibc
build_kernel
build_busybox
build_strace
build_zlib
build_ncurses
build_ftpd
build_joe
build_gdbserver
build_dropbear
build_ntp
build_net_tools
build_dnsmasq
create_etc_release
create_image

# Build STK1000 Standard Linux image
ROOT=${ROOT_STK1000}
ROOT_REF=${ROOT_BASIC}
ROOT_IMAGE_NAME="STK1000 Linux image - standard"
ROOT_IMAGE_SIZE=120000
create_root
build_libpng
build_libungif
build_libjpeg
build_fbv
build_fbset
build_alsa_lib
build_alsa_utils
build_fusion
build_freetype
build_directfb
build_pong
build_gawk
get_stk1000_web
create_etc_release
create_image


end_timer
report_time
exit 0
